@@ -4,6 +4,7 @@ Helper methods for generating k8s API objects.
 import base64
 import ipaddress
 import json
+import operator
 import os
 import re
 from urllib.parse import urlparse
@@ -402,9 +403,9 @@ def make_pod(
     # populate with fs_gid / supplemental_gids
     if fs_gid is not None:
         psc["fsGroup"] = int(fs_gid)
-    if supplemental_gids is not None:
+    if supplemental_gids:
         psc["supplementalGroups"] = [int(gid) for gid in supplemental_gids]
-    if pod_security_context is not None:
+    if pod_security_context:
         for key in pod_security_context.keys():
             if "_" in key:
                 raise ValueError(
@@ -425,7 +426,7 @@ def make_pod(
         csc["privileged"] = True
     if not allow_privilege_escalation:  # true as default
         csc["allowPrivilegeEscalation"] = False
-    if container_security_context is not None:
+    if container_security_context:
         for key in container_security_context.keys():
             if "_" in key:
                 raise ValueError(
@@ -435,24 +436,79 @@ def make_pod(
     if not csc:
         csc = None
 
-    # Transform a dict into valid Kubernetes EnvVar Python representations. This
-    # representation shall always have a "name" field as well as either a
-    # "value" field or "value_from" field. For examples see the
-    # test_make_pod_with_env function.
-    prepared_env = []
-    for k, v in (env or {}).items():
-        if type(v) == dict:
-            if not "name" in v:
-                v["name"] = k
-            prepared_env.append(get_k8s_model(V1EnvVar, v))
+    def _get_env_var_deps(env):
+        # only consider env var objects with an explicit string value
+        if not env.value:
+            return set()
+        # $(MY_ENV) pattern: $( followed by non-)-characters to be captured, followed by )
+        re_k8s_env_reference_pattern = r"\$\(([^\)]+)\)"
+        deps = set(re.findall(re_k8s_env_reference_pattern, env.value))
+        return deps - {env.name}
+
+    unsorted_env = {}
+    for key, env in (env or {}).items():
+        # Normalize KubeSpawners env input to valid Kubernetes EnvVar Python
+        # representations. They should have a "name" field as well as either a
+        # "value" field or "value_from" field. For examples see the
+        # test_make_pod_with_env function.
+        if type(env) == dict:
+            if not "name" in env:
+                env["name"] = key
+            env = get_k8s_model(V1EnvVar, env)
         else:
-            prepared_env.append(V1EnvVar(name=k, value=v))
+            env = V1EnvVar(name=key, value=env)
+
+        # Extract information about references to other envs as we want to use
+        # those to make an intelligent sorting before we render this into a list
+        # with an order that matters.
+        unsorted_env[env.name] = {
+            "deps": _get_env_var_deps(env),
+            "key": key,
+            "env": env,
+        }
+
+    # We sort environment variables in a way that allows dependencies to other
+    # env to resolve as much as possible. There could be circular dependencies
+    # so we will just do our best and settle with that.
+    #
+    # Algorithm description:
+    #
+    # - loop step:
+    #   - pop all unsorted_env entries with dependencies in sorted_env
+    #   - sort popped env based on key and extend the sorted_env list
+    # - loop exit:
+    #   - exit if loop step didn't pop anything from unsorted_env
+    #   - before exit, sort what remains and extending the sorted_env list
+    #
+    sorted_env = []
+    while True:
+        already_resolved_env_names = [e.name for e in sorted_env]
+
+        extracted_env = {}
+        for k, v in unsorted_env.copy().items():
+            if v["deps"].issubset(already_resolved_env_names):
+                extracted_env[k] = unsorted_env.pop(k)
+
+        if extracted_env:
+            extracted_env = [
+                d["env"]
+                for d in sorted(extracted_env.values(), key=operator.itemgetter("key"))
+            ]
+            sorted_env.extend(extracted_env)
+        else:
+            remaining_env = [
+                d["env"]
+                for d in sorted(unsorted_env.values(), key=operator.itemgetter("key"))
+            ]
+            sorted_env.extend(remaining_env)
+            break
+
     notebook_container = V1Container(
         name='notebook',
         image=image,
         working_dir=working_dir,
         ports=[V1ContainerPort(name='notebook-port', container_port=port)],
-        env=prepared_env,
+        env=sorted_env,
         args=cmd,
         image_pull_policy=image_pull_policy,
         lifecycle=lifecycle_hooks,
