@@ -20,7 +20,6 @@ from .utils import generate_hashed_slug
 
 class IngressReflector(ResourceReflector):
     kind = 'ingresses'
-    list_method_name = 'list_namespaced_ingress'
     api_group_name = 'ExtensionsV1beta1Api'
 
     @property
@@ -30,7 +29,6 @@ class IngressReflector(ResourceReflector):
 
 class ServiceReflector(ResourceReflector):
     kind = 'services'
-    list_method_name = 'list_namespaced_service'
 
     @property
     def services(self):
@@ -39,7 +37,6 @@ class ServiceReflector(ResourceReflector):
 
 class EndpointsReflector(ResourceReflector):
     kind = 'endpoints'
-    list_method_name = 'list_namespaced_endpoints'
 
     @property
     def endpoints(self):
@@ -114,6 +111,12 @@ class KubeIngressProxy(Proxy):
         # an accidental bottleneck. Since we serialize our create operations, we only
         # need 1x concurrent_spawn_limit, not 3x.
         self.executor = ThreadPoolExecutor(max_workers=self.app.concurrent_spawn_limit)
+
+        # Global configuration before reflector.py code runs
+        self._set_k8s_client_configuration()
+        self.core_api = shared_client('CoreV1Api')
+        self.extension_api = shared_client('ExtensionsV1beta1Api')
+
         labels = {
             'component': self.component_label,
             'hub.jupyter.org/proxy-route': 'true',
@@ -128,11 +131,6 @@ class KubeIngressProxy(Proxy):
             parent=self, namespace=self.namespace, labels=labels
         )
 
-        # Global configuration before reflector.py code runs
-        self._set_k8s_client_configuration()
-        self.core_api = shared_client('CoreV1Api')
-        self.extension_api = shared_client('ExtensionsV1beta1Api')
-
     def _set_k8s_client_configuration(self):
         # The actual (singleton) Kubernetes client will be created
         # in clients.py shared_client but the configuration
